package a;

import javax.media.opengl.GL;

import zet.graphics.ogl.grid.Grid;
import zet.graphics.ogl.grid.InfiniteGrid;
import zet.graphics.ogl.util.OGLDrawTools;
import zet.graphics.rayz.core.IntersectionData;
import zet.graphics.rayz.core.Ray;
import zet.graphics.rayz.core.RayConsts;
import zet.graphics.rayz.core.RayScene;
import zet.graphics.rayz.shapes.RayShape;
import zet.graphics.rayz.tree.Node;
import zet.graphics.rayz.tree.Tree;
import zet.math.Vector3d;
import zet.tred.zengine.view.ViewPoint;

/**
 *
 * @author Mattias
 */
public class OpenGLScene {

	private RayScene scene;
	private Grid grid;
	private boolean lightning = true;

	public OpenGLScene(RayScene scene, ViewPoint vp) {
		this.scene = scene;

		grid = new InfiniteGrid(vp);
	}

	public void draw(GL glay) {
		GL gl = glay;

		grid.draw(glay);

		if (lightning) {
			gl.glEnable(GL.GL_LIGHTING);
		}
		for (RayShape shape : scene.getShapes()) {
		//for (RayShape shape : scene.getTree().getSuperNode().getPrimitives()) {
			Vector3d color = shape.getMaterial().getColor();
			gl.glColor3d(color.x, color.y, color.z);
			ShapeDrawer.drawShape(gl, shape.getShape());
		}
		drawTree(glay, scene.getTree());

		gl.glDisable(GL.GL_LIGHTING);

		double h = Math.sin(Math.toRadians(RayConsts.FOVY / 2)) * RayConsts.FOCUSLENGTH;
		double w = h * RayConsts.WIDTH / RayConsts.HEIGHT;

		drawCamera(gl, w / RayConsts.FOCUSLENGTH, h / RayConsts.FOCUSLENGTH);
		drawCameraRay(gl, 0, 0, true);
		drawRayPath(gl, scene.getLightRay(0, 0));
		boolean drawCameraRay = false;
		boolean drawCameraDrawRay = false;
		boolean drawRayPath = false;
		if (scene.getViewPoint().getPosition().distance() != 0 && false) {
			double pw = 40.0f * 1, ph = 40.0f * 1;
			double a = w / pw * 2;
			double b = h / ph * 2;

			for (double i = -w; i < w; i += a) {
				for (double j = -h; j < h; j += b) {
					if (drawRayPath) {
						drawRayPath(gl, scene.getLightRay(i, j));
					}
					if (drawCameraRay) {
						drawCameraRay(gl, i, j, drawCameraDrawRay);
					}

					Vector3d c = scene.getPixelColor(i, j);
					gl.glColor3d(c.x, c.y, c.z);

					if (c == null) {
						gl.glBegin(GL.GL_QUADS);

						OGLDrawTools.drawVertex(gl, scene.getPixelPoint(i / RayConsts.FOCUSLENGTH, j / RayConsts.FOCUSLENGTH));
						OGLDrawTools.drawVertex(gl, scene.getPixelPoint((i + a) / RayConsts.FOCUSLENGTH, j / RayConsts.FOCUSLENGTH));
						OGLDrawTools.drawVertex(gl, scene.getPixelPoint((i + a) / RayConsts.FOCUSLENGTH, (j + b) / RayConsts.FOCUSLENGTH));
						OGLDrawTools.drawVertex(gl, scene.getPixelPoint(i / RayConsts.FOCUSLENGTH, (j + b) / RayConsts.FOCUSLENGTH));

						gl.glEnd();
					}
				}
				if (drawCameraRay) {
					drawCameraRay(gl, i, h, drawCameraDrawRay);
				}
				if (drawRayPath) {
					drawRayPath(gl, scene.getLightRay(i, h));
				}
			}
			for (double j = -h; j < h; j += b) {
				if (drawCameraRay) {
					drawCameraRay(gl, w, j, drawCameraDrawRay);
				}
				if (drawRayPath) {
					drawRayPath(gl, scene.getLightRay(w, j));
				}
			}
		}
	}

	public void drawRayPath(GL glay, final Ray ray) {
		GL gl = glay;

		Ray curRay = ray;

		int depth = 0;
		IntersectionData id;
		gl.glBegin(GL.GL_LINES);
		do {
			depth++;
			id = scene.getIntersection(curRay);
			if (id != null) {
				gl.glColor3f(1, 1, 1);

				if (id.pos != null) {
					if (id.i != null) { // i
						OGLDrawTools.drawVertex(gl, id.i.o);
						OGLDrawTools.drawVertex(gl, id.pos);
					}

					if (id.getNormal() != null) { // n
						OGLDrawTools.drawVertex(gl, id.pos);
						OGLDrawTools.drawVertex(gl, id.getNormal().add(id.pos));
					}

					if (!id.shape.getMaterial().isLightSource()) {
						if (id.r != null) { // reflection
							OGLDrawTools.drawVertex(gl, id.r.o);
							OGLDrawTools.drawVertex(gl, id.r.d.mul(5).add(id.r.o));

							curRay = id.r;
						}
						if (id.refraction != null) { // refraction
							OGLDrawTools.drawVertex(gl, id.refraction.o);
							OGLDrawTools.drawVertex(gl, id.refraction.d.mul(5).add(id.refraction.o));

							//	curRay = id.refraction;
						} else {
							break;
						}
					} else {
						break;
					}
				} else {
					break;
				}
			} else {
				break;
			}
		} while (depth < 10);
		gl.glEnd();
	}

	public void drawCameraRay(GL glay, double w, double h, boolean aas) {
		GL gl = glay;

		Vector3d pp, fp, pd;
		pp = scene.getPixelPoint(w, h);
		fp = scene.getViewPoint().getFocusPoint().add(scene.getViewPoint().getPosition());
		pd = pp.sub(fp).normalize();

		if (aas) {
			gl.glBegin(GL.GL_LINES);
		}

		gl.glColor3f(0.8f, 0.8f, 0.8f);
		IntersectionData id = scene.getIntersection(new Ray(pp, pd));
		if (id != null) {
			Vector3d sn = id.pos.sub(id.shape.getPosition()).normalize();

			gl.glColor3d(1 - (Vector3d.angle(sn.normalize(), pd.negate().normalize()) / (Math.PI / 2f)), 0.0f, 0.0f);
		}
		if (aas) {
			gl.glVertex3d(pp.x, pp.y, pp.z);
			gl.glVertex3d(pp.x + pd.x * 1000, pp.y + pd.y * 1000, pp.z + pd.z * 1000);

			gl.glEnd();
		}
	}

	public void drawCamera(GL glay, double w, double h) {
		GL gl = glay;

		Vector3d pos = scene.getViewPoint().getPosition();
		Vector3d[] dirs = scene.getViewPoint().getDirections();
		Vector3d fv = scene.getViewPoint().getFocusPoint().add(pos);
		Vector3d dx = dirs[0];
		Vector3d dy = dirs[1];
		Vector3d dz = dirs[2];

		gl.glBegin(GL.GL_LINES);

		gl.glColor3f(1.0f, 0.0f, 0.0f);
		gl.glVertex3d(pos.x, pos.y, pos.z);
		gl.glVertex3d(pos.x + dz.x, pos.y, pos.z);

		gl.glColor3f(0.0f, 1.0f, 0.0f);
		gl.glVertex3d(pos.x, pos.y, pos.z);
		gl.glVertex3d(pos.x, pos.y + dz.y, pos.z);

		gl.glColor3f(0.0f, 0.0f, 1.0f);
		gl.glVertex3d(pos.x, pos.y, pos.z);
		gl.glVertex3d(pos.x, pos.y, pos.z + dz.z);

		if (true) {
			gl.glColor3f(0.0f, 1.0f, 1.0f);
			gl.glVertex3d(pos.x, pos.y, pos.z);
			gl.glVertex3d(dx.x + pos.x, dx.y + pos.y, dx.z + pos.z);

			gl.glColor3f(1.0f, 1.0f, 0.0f);
			gl.glVertex3d(pos.x, pos.y, pos.z);
			gl.glVertex3d(dy.x + pos.x, dy.y + pos.y, dy.z + pos.z);
		}

		gl.glColor3f(1.0f, 0.0f, 1.0f);
		Vector3d[] vs = {
			dx.mul(w).add(new Vector3d(dy.mul(h)).add(pos)),
			dx.mul(w).add(new Vector3d(dy.mul(-h)).add(pos)),
			dx.mul(-w).add(new Vector3d(dy.mul(-h)).add(pos)),
			dx.mul(-w).add(new Vector3d(dy.mul(h)).add(pos))
		};

		if (!true) {
			gl.glColor3f(1.0f, 1.0f, 1.0f);
			gl.glVertex3d(fv.x, fv.y, fv.z);
			gl.glVertex3d(fv.x + dz.x * 1000, fv.y + dz.y * 1000, fv.z + dz.z * 1000);
		}

		gl.glEnd();

		gl.glBegin(GL.GL_LINE_LOOP);
		gl.glColor3f(1.0f, 0.0f, 1.0f);
		for (Vector3d vector : vs) {
			gl.glVertex3d(vector.x, vector.y, vector.z);
			gl.glVertex3d(fv.x, fv.y, fv.z);
			gl.glVertex3d(vector.x, vector.y, vector.z);
		}
		gl.glEnd();
	}

	public void drawTree(GL gl, Tree tree) {
		drawNode(gl, tree.getSuperNode());
	}

	public void drawNode(GL gl, Node node) {
		if (node == null) {
			return;
		}
		Ray r = scene.getLightRay(0, 0);
		Vector3d pos = r.o.add(r.d.mul(10));
		//pos = r.o;
		if (!node.contains(pos) && false) {
			return;
		}
		if (node.intersects(r) == null) {
			return;
		}

		if (node.left == null && node.right == null || true) {
			gl.glPushAttrib(GL.GL_COLOR_MATRIX);
			gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_LINE);

			gl.glColor3d(1, 1, 1);
			OGLDrawTools.drawBoundingBox(gl, node.getBoundingBox());

			gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
			gl.glPopAttrib();
		}

		gl.glColor3d(1, 0, 0);
		drawNode(gl, node.left);
		gl.glColor3d(0, 1, 0);
		drawNode(gl, node.right);
	}
}
