package a;

import com.sun.opengl.util.Animator;
import com.sun.opengl.util.GLUT;
import com.sun.opengl.util.Screenshot;
import java.awt.Dimension;
import java.awt.Image;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;
import zet.graphics.ogl.util.OGLTextDrawer;
import zet.graphics.rayz.core.RayConsts;
import zet.graphics.rayz.core.RayListener;
import zet.graphics.rayz.core.RayScene;
import zet.tred.zengine.view.Camera;
import zet.tred.zengine.view.MouseGrip;
import zet.tred.zengine.view.ViewPointMovement;

/**
 *
 * @author Mattias
 */
public class Ragly implements GLEventListener {

	private Dimension size;
	private Animator animator;
	private Camera camera;
	private OpenGLScene scene;
	private MouseGrip mg;
	private Image screenshot;
	private boolean started;
	private long time;
	private int frames;
	private int fps;

	public Ragly(RayPanel rp, final RayScene scene, final RayListener listener) {
		size = rp.getSize();
		animator = new Animator(rp);

		camera = new Camera();
		camera.setPosition(scene.getViewPoint().getPosition());
		camera.setDirection(scene.getViewPoint().getDirection());

		mg = new MouseGrip(rp, camera, MouseGrip.MouseGripMode.CENTERED);
		rp.addMouseListener(mg);
		rp.addMouseMotionListener(mg);
		ViewPointMovement.WASD.setViewPoint(camera);
		rp.addKeyListener(ViewPointMovement.WASD);

		scene.setViewPoint(camera.getPosition(), camera.getDirection());

		this.scene = new OpenGLScene(scene, camera);
		start(RayConsts.cameraModification);
	}

	public void start() {
		start(true);
	}

	public void start(boolean cm) {
		animator.start();
		started = true;
		mg.setGrip(cm);
	}

	public void stop() {
		animator.stop();
		started = false;
		mg.setGrip(started);
	}

	public boolean isStarted() {
		return started;
	}

	public Image getScreenshot() {
		return screenshot;
	}

	public int getWidth() {
		return size.width;
	}

	public int getHeight() {
		return size.height;
	}

	public Camera getCamera() {
		return camera;
	}

	public void display(GLAutoDrawable glad) {
		final GL gl = glad.getGL();

		gl.glClear(GL.GL_COLOR_BUFFER_BIT);
		gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
		gl.glLoadIdentity();

		camera.rotate(gl);
		camera.translate(gl);

		scene.draw(gl);

		if (System.currentTimeMillis() - time > 1000) {
			fps = frames;
			frames = 0;
			time = System.currentTimeMillis();

			screenshot = Screenshot.readToBufferedImage(getWidth(), getHeight());
		}
		OGLTextDrawer.drawText(gl, "FPS: " + fps, GLUT.BITMAP_HELVETICA_12, 10, 10);
		OGLTextDrawer.drawText(gl, "Position: (" + camera.getX() + ", " + camera.getY() + ", " + camera.getZ() + "), " + "distance: " + Math.sqrt(camera.getX() * camera.getX() + camera.getY() * camera.getY() + camera.getZ() * camera.getZ()),
				GLUT.BITMAP_HELVETICA_12, 10, 10 + 5 + 12);
		OGLTextDrawer.drawText(gl, "Direction: " + camera.getDirection(),
				GLUT.BITMAP_HELVETICA_12, 10, 10 + (5 + 12) * 2);
		frames++;
	}

	public void init(GLAutoDrawable glad) {
		GL gl = glad.getGL();
		gl.glShadeModel(GL.GL_SMOOTH);
		gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		gl.glClearDepth(1.0f);
		gl.glEnable(GL.GL_DEPTH_TEST);
		gl.glDepthFunc(GL.GL_LEQUAL);
		gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);

		gl.glEnable(GL.GL_FOG);
		gl.glFogi(GL.GL_FOG_MODE, GL.GL_EXP2);
		gl.glFogf(GL.GL_FOG_DENSITY, 0.015f);
		gl.glHint(GL.GL_FOG_HINT, GL.GL_DONT_CARE);
		gl.glFogf(GL.GL_FOG_START, 100.0f); // Fog Start Depth
		gl.glFogf(GL.GL_FOG_END, 350.0f); // Fog End Depth

		float[] lightAmbient = {0.5f, 0.5f, 0.5f, 1.0f};
		float[] lightDiffuse = {1.0f, 1.0f, 1.0f, 1.0f};
		float[] lightPosition = {0.0f, 0.0f, 0.0f, 1.0f};
		gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, lightAmbient, 0);
		gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, lightDiffuse, 0);
		gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, lightPosition, 0);
		gl.glEnable(GL.GL_LIGHT0);
		gl.glEnable(GL.GL_LIGHTING);

		gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE);

		gl.glEnable(GL.GL_COLOR_MATERIAL);
	}

	public void dispose(@SuppressWarnings("unused") GLAutoDrawable glad) {
		// Not used.
	}

	public void displayChanged(GLAutoDrawable glad, boolean arg1, boolean arg2) {
		throw new UnsupportedOperationException("Not supported yet.");
	}

	public void reshape(GLAutoDrawable glad, int x, int y, int width, int height) {
		size.width = width;
		size.height = height;

		if (glad != null) {
			final GL gl = glad.getGL();
			final GLU glu = new GLU();

			gl.glViewport(0, 0, width, height);
			gl.glMatrixMode(GL.GL_PROJECTION);
			gl.glLoadIdentity();

			glu.gluPerspective(RayConsts.FOVY, (double) width / height, RayConsts.NEAR, RayConsts.FAR);
			gl.glMatrixMode(GL.GL_MODELVIEW);
			gl.glLoadIdentity();
		}
	}
}
