package a;

import java.awt.CardLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import javax.script.ScriptException;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import zet.graphics.rayz.core.IntersectionData;
import zet.graphics.rayz.core.Perspective;
import zet.graphics.rayz.core.PerspectiveType;
import zet.graphics.rayz.core.RayConsts;

import zet.graphics.rayz.shapes.GroupShape;
import zet.graphics.rayz.core.RayMaterial;
import zet.graphics.rayz.core.RayScene;
import zet.graphics.rayz.core.Scene;
import zet.graphics.rayz.math.RayMath;
import zet.graphics.rayz.opengl.shapes.Model;
import zet.graphics.rayz.script.RayScript;
import zet.graphics.rayz.tracer.RayImage;
import zet.graphics.shapes.Ball;
import zet.graphics.shapes.Cuboid;
import zet.graphics.shapes.Plane;
import zet.graphics.shapes.Sphere;
import zet.math.Vector3d;
import zet.math.VectorMath3d;

/**
 *
 * @author Mattias
 */
public class Raywin {

	public static final String TITLE = "OpenRW";
	public static boolean externalRendering = false;
	private RayPanel preview = null;

	public Raywin() {
		final JFrame frame = new JFrame(Raywin.TITLE);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		final RayScene scene = new RayScene();

		initScene(scene);
		scene.done();


		final CardLayout cl = new CardLayout();
		frame.setLayout(cl);
		preview = new RayPanel(frame, scene);
		final Ragly ragly = preview.getRagly();

		frame.add(preview, "Preview");
		final RayImage rayImage = new RayImage(frame, scene, RayConsts.WIDTH, RayConsts.HEIGHT);
		frame.add(rayImage, "Render");
		cl.show(preview.getParent(), "Preview");
		frame.pack();

		frame.addKeyListener(new KeyAdapter() {

			@Override
			public void keyPressed(KeyEvent e) {
				super.keyPressed(e);

				if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					if (ragly.isStarted()) {
						ragly.stop();

						frame.setTitle(Raywin.TITLE + " - OpenGL (paused)");
					} else {
						scene.stop();
						ragly.start();

						cl.show(preview.getParent(), "Preview");
						preview.requestFocus();
						frame.setTitle(Raywin.TITLE + " - OpenGL");
					}
				} else {
					if (e.getKeyCode() == KeyEvent.VK_SPACE || e.getKeyCode() == KeyEvent.VK_M) {
						scene.setViewPoint(preview.getPosition(), preview.getDirection());

						ShapeDrawer.vp = scene.getViewPoint();

						if (e.getKeyCode() == KeyEvent.VK_SPACE) {
							rayImage.setBackgroundImage(ragly.getScreenshot());
							if (ragly.isStarted()) {
								ragly.stop();
							}
							scene.stop();
							scene.raytrace(rayImage);
							cl.show(preview.getParent(), "Render");
							rayImage.requestFocus();
						}
					}
				}
			}
		});
		for (KeyListener listener : frame.getKeyListeners()) {
			preview.addKeyListener(listener);
			rayImage.addKeyListener(listener);
		}

		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
		preview.requestFocus();
	}

	public void initScene(RayScene scene) {
		Perspective perspective = new Perspective(PerspectiveType.Conic);
		scene.setPerspective(perspective);
		scene.setComplexity(1);

		int sceneID = -2;
		if (sceneID == -2) {
			try {
				//RayScript.load(scene, "Scenes/Scene.js");
				//RayScript.load(scene, "Scenes/Devmaster3.js");
				//RayScript.load(scene, "Scenes/SphereAndBox.js");
				RayScript.load(scene, "Scenes/Bump.js");
			} catch (IOException ex) {
				ex.printStackTrace();
				System.exit(0);
			} catch (ScriptException ex) {
				ex.printStackTrace();
				System.exit(0);
			}
		} else if (sceneID == -1) {
			scene.addShape(new Sphere(5, 10, 0, 2));
			scene.addShape(new Sphere(0, 25, 5, 0.5), RayMaterial.LIGHTSOURCE);
			//scene.addShape(new Sphere(0, 25, -5, 0.5), RayMaterial.LIGHTSOURCE);
			//scene.addShape(new Sphere(5, 25, 0, 0.5), RayMaterial.LIGHTSOURCE);
			scene.addShape(new Sphere(-5, 25, 0, 0.5), RayMaterial.LIGHTSOURCE);
			//scene.addShape(new Model(new Vector3d(0, 5, 0), "Models/cube.obj"));
			//scene.addShape(new Model(new Vector3d(0, 0, 0), "Models/teapot.obj"));
			//scene.addShape(new Model(new Vector3d(0, 0, 0), "Models/cessna.obj"));
			//scene.addShape(new Model(new Vector3d(0, 0, 0), "Models/mario.obj"));
			scene.addShape(new Model(new Vector3d(0, 5, 0), "Models/PSG1.obj"));

			addRoom(scene, 30);
		} else if (sceneID == 0) {
			RayMaterial.LIGHTSOURCE.setColor(.8, 1, .8);
			//	scene.addShape(new Model(new Vector3d(-5, 5, 0), "Models/cessna.obj"));
			scene.addShape(new Sphere(-5, 25, 5, 0.5), RayMaterial.LIGHTSOURCE);
			scene.addShape(new Sphere(5, 25, -5, 0.5), RayMaterial.LIGHTSOURCE);
			scene.addShape(new Sphere(0, 5, 0, 2));

			//	scene.addShape(new Quad(new Vector3d(0, 15, -5), new Vector3d(0, -1, 0), 4, 3), RayMaterial.LIGHTSOURCE);
			scene.addShape(new Cuboid(new Vector3d(2, 8, 2), new Vector3d(3, 1, 2)));
			//scene.addShape(new Cylinder(new Vector3d(0, 8, 0), new Vector3d(0, 0, 1), 4, 2));

			for (int i = 0; i < 50; i++) {
				scene.addShape(new Ball(5, 15, 0, Math.random() + 1));
			}

			addRoom(scene, 30);

			scene.addShape(new GroupShape(new Vector3d(10, -5, -10)) {

				@Override
				public void init() {
					addShape(new Sphere(0, 5, 5, 2), RayMaterial.MIRROR);
					addShape(new Sphere(0, 15, 0, 2), RayMaterial.GLASS);
				}
			});
		}
	}

	public static void addRoom(Scene scene, final double size) {
		scene.addShape(new GroupShape(new Vector3d(0, 0, 0)) {

			@Override
			public void init() {
				RayMaterial checker = new RayMaterial() {

					@Override
					public Vector3d getColor(IntersectionData data) {
						return getChecker(data.getPosition(), data.getOriginalNormal());
					}

					public Vector3d getChecker(Vector3d pos, Vector3d n) {
						double c = 1;
						if (n != null) {
							Vector3d ua = new Vector3d(n.y, n.z, -n.x).normalize();
							Vector3d va = VectorMath3d.cross(ua, n).normalize();

							pos = pos.add(n.mul(RayConsts.EPSILON));
							double u = pos.dot(ua);
							double v = pos.dot(va);

							double size = 1;
							double q = (q = u % (size * 2)) >= 0 ? q : size * 2 + q;
							double w = (w = v % (size * 2)) >= 0 ? w : size * 2 + w;

							c = (q > size / 1) ^ (w > size) ? 1 : 0;
						}

						return super.getColor().mul(c);
					}
				};

				RayMaterial.SOLID.setColor(0.5, 0.5, 0.5);
				addShape(new Plane(new Vector3d(0, 0, 0), new Vector3d(0, 1, 0)), checker);
				addShape(new Plane(new Vector3d(0, size, 0), new Vector3d(0, -1, 0)), checker);
				addShape(new Plane(new Vector3d(-size, 0, 0), new Vector3d(1, 0, 0)), checker);
				addShape(new Plane(new Vector3d(size, 0, 0), new Vector3d(-1, 0, 0)), checker);
				addShape(new Plane(new Vector3d(0, 0, -size), new Vector3d(0, 0, 1)), checker);
				addShape(new Plane(new Vector3d(0, 0, size), new Vector3d(0, 0, -1)), checker);
			}
		});
	}

	public static void main(String[] args) {
		System.out.println("Starting...");
		Vector3d l = new Vector3d(0.707107, -0.707107, 0);
		Vector3d n = new Vector3d(0, 1, 0);

		Vector3d r = RayMath.refract(l, n, 0.9);

		System.out.println(r);

		SwingUtilities.invokeLater(new Runnable() {

			public void run() {
				System.out.println("Creating window...");
				new Raywin();
			}
		});
	}
}
