package a;

import com.sun.opengl.impl.GLUquadricImpl;
import com.sun.opengl.util.GLUT;
import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;
import zet.graphics.ogl.util.OGLDrawTools;
import zet.graphics.rayz.core.IntersectionData;
import zet.graphics.rayz.core.Ray;
import zet.graphics.rayz.core.ViewPoint;
import zet.graphics.rayz.math.RayMath;
import zet.math.Vector3d;
import zet.graphics.shapes.Ball;
import zet.graphics.shapes.Cuboid;
import zet.graphics.shapes.Cylinder;
import zet.graphics.rayz.shapes.GroupShape;
import zet.graphics.rayz.shapes.RayShape;
import zet.graphics.rayz.opengl.shapes.Model;
import zet.graphics.rayz.shapes.CSGShape;
import zet.graphics.shapes.Blob;
import zet.graphics.shapes.BoundingBox;
import zet.graphics.shapes.Ellipsoid;

import zet.graphics.shapes.Plane;
import zet.graphics.shapes.Primitive;
import zet.graphics.shapes.Quad;
import zet.graphics.shapes.Shape;
import zet.graphics.shapes.Sphere;
import zet.graphics.shapes.Triangle;
import zet.graphics.shapes.UnknownShapeException;

/**
 *
 * @author Mattias
 */
public class ShapeDrawer {

	private static GLUT glut = new GLUT();
	private static GLU glu = new GLU();
	private static int sub = 16;
	public static ViewPoint vp;

	public static void setSubdivision(int sub) {
		ShapeDrawer.sub = sub;
	}

	public static void drawShape(GL glay, Shape shape) {
		drawShape(glay, shape, true);
	}

	public static void drawShape(GL glay, Shape shape, boolean drawBox) {
		GL gl = glay;

		if (drawBox && shape.getBoundingBox() != null) {
			gl.glPushMatrix();
			gl.glTranslated(shape.getX(), shape.getY(), shape.getZ());
			drawShape(glay, shape.getBoundingBox(), false);
			gl.glPopMatrix();
		}

		gl.glPushMatrix();
		gl.glTranslated(shape.getX(), shape.getY(), shape.getZ());
		gl.glRotated(shape.getRotation().x, 1, 0, 0);
		gl.glRotated(shape.getRotation().y, 0, 1, 0);
		gl.glRotated(shape.getRotation().z, 0, 0, 0);

		if (!(shape instanceof Primitive)) {
			if (shape instanceof BoundingBox) {
				gl.glPushAttrib(GL.GL_COLOR_MATRIX);
				gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_LINE);
				gl.glColor3f(1, 1, 1);
				OGLDrawTools.drawBoundingBox(gl, (BoundingBox) shape);
				gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
				gl.glPopAttrib();
			} else if (shape instanceof Model) {
				gl.glDisable(GL.GL_COLOR_MATERIAL);
				((Model) shape).draw(gl);
				gl.glEnable(GL.GL_COLOR_MATERIAL);
			} else if (shape instanceof GroupShape) {
				gl.glPushMatrix();
				for (RayShape s : ((GroupShape) shape).getShapes()) {
					Vector3d color = s.getMaterial().getColor();
					gl.glColor3d(color.x, color.y, color.z);
					drawShape(glay, s.getShape(), false);
				}
				gl.glPopMatrix();
			} else if (shape instanceof CSGShape) {
				drawShape(glay, ((CSGShape) shape).getA());
				drawShape(glay, ((CSGShape) shape).getB());
			} else if (shape instanceof Cuboid) {
				double x = ((Cuboid) shape).getSize().x;
				double y = ((Cuboid) shape).getSize().y;
				double z = ((Cuboid) shape).getSize().z;

				OGLDrawTools.drawCuboid(gl, x, y, z);
			} else {
				throw new UnknownShapeException("Shape: " + shape.getClass().getName());
			}
		} else {
			if (shape instanceof Primitive) {
				drawPrimitive(gl, shape);
			}
		}

		if (!true) {
			IntersectionData id;
			if (vp != null && (id = shape.intersects(new Ray(vp.getPosition(), vp.getDirection()))) != null) {
				if (false) {
					gl.glPopMatrix();
					gl.glPushMatrix();
					gl.glColor3f(1, 1, 1);
					Vector3d i = vp.getDirection().normalize();
					Vector3d vl = vp.getPosition().add(vp.getDirection().mul(1));
					Vector3d v = vp.getPosition().add(vp.getDirection().mul(id.distance));
					Vector3d n = v.sub(shape.getPosition()).normalize();
					Vector3d vec = vp.getDirection().normalize().negate();

					gl.glBegin(GL.GL_LINES);
					/*
					gl.glVertex3f(shape.getX(), shape.getY(), shape.getZ());
					gl.glVertex3f(v.x, v.y, v.z);

					gl.glVertex3f(shape.getX(), shape.getY(), shape.getZ());
					gl.glVertex3f(vl.x, vl.y, vl.z);*/

					gl.glVertex3d(v.x, v.y, v.z);
					gl.glVertex3d(n.x + v.x, n.y + v.y, n.z + v.z);

					gl.glVertex3d(v.x, v.y, v.z);
					gl.glVertex3d(vec.x + v.x, vec.y + v.y, vec.z + v.z);

					Vector3d r = n.mul(n.dot(i)).mul(2).sub(i).negate();
					gl.glVertex3d(v.x, v.y, v.z);
					gl.glVertex3d(r.x + v.x, r.y + v.y, r.z + v.z);

					gl.glEnd();
					gl.glTranslated(v.x, v.y, v.z);
					glut.glutWireSphere(((Sphere) shape).getRadie() / 6, sub, sub);
				}
			}
		}
		gl.glPopMatrix();

		if (shape instanceof Ball) {
			((Ball) shape).getDirection().setY(((Ball) shape).getDirection().y - 0.00982f); // Acceleration (gravitation)
			((Ball) shape).getDirection().set(((Ball) shape).getDirection().mul(0.999f)); // Air resistance
			shape.getPosition().set(shape.getPosition().add(((Ball) shape).getDirection())); // Move

			if (shape.getY() - ((Sphere) shape).getRadie() < 0) { // Blob is under ground?
				double speed = ((Ball) shape).getDirection().distance();
				((Ball) shape).getDirection().set(RayMath.reflect(((Ball) shape).getDirection(), new Vector3d(0, 1, 0)));
				((Ball) shape).getDirection().set(((Ball) shape).getDirection().mul(speed));
				((Ball) shape).getDirection().setY(((Ball) shape).getDirection().y * 0.9f); // Energy loss
				((Ball) shape).getDirection().set(((Ball) shape).getDirection().mul(0.95f)); // Friction
				shape.getPosition().setY(((Sphere) shape).getRadie()); // Move over ground.
			}
		}
	}

	private static <S extends Shape> void drawPrimitive(GL gl, S shape) {
		if (shape instanceof Sphere) {
			glut.glutSolidSphere(((Sphere) shape).getRadie(), sub, sub);
		} else if (shape instanceof Ellipsoid) {
			glut.glutWireSphere(((Ellipsoid) shape).getRadiis().distance(), sub, sub);
		} else if (shape instanceof Triangle) {
			gl.glBegin(GL.GL_TRIANGLES);

			gl.glNormal3d(((Triangle) shape).getNormal().x, ((Triangle) shape).getNormal().y, ((Triangle) shape).getNormal().z);

			OGLDrawTools.drawVertex(gl, ((Triangle) shape).getA());
			OGLDrawTools.drawVertex(gl, ((Triangle) shape).getB());
			OGLDrawTools.drawVertex(gl, ((Triangle) shape).getC());

			gl.glEnd();
		} else if (shape instanceof Quad) {
			gl.glBegin(GL.GL_QUADS);

			gl.glNormal3d(((Quad) shape).getDz().x, ((Quad) shape).getDz().y, ((Quad) shape).getDz().z);
			OGLDrawTools.drawPolygon(gl, ((Quad) shape).getPoints());

			gl.glEnd();
		} else if (shape instanceof Plane) {
			gl.glBegin(GL.GL_QUADS);

			gl.glNormal3d(((Plane) shape).getDx().x, ((Plane) shape).getDx().y, ((Plane) shape).getDx().z);

			final double w = 1000, h = w;
			OGLDrawTools.drawQuad(gl, shape.getPosition(), ((Plane) shape).getDx(), ((Plane) shape).getDy(), w, h);

			gl.glEnd();
		} else if (shape instanceof Cylinder) {
			GLUquadric gluq = new GLUquadricImpl();
			Vector3d dir = ((Cylinder) shape).getDirection();

			double rx = -Math.toDegrees(Math.asin(dir.y));
			double ry = -Math.toDegrees(Math.atan(dir.x / -dir.z));

			gl.glRotated(rx, 1.0f, 0.0f, 0.0f);
			gl.glRotated(ry, 0.0f, 1.0f, 0.0f);
			glu.gluCylinder(gluq, ((Cylinder) shape).getRadie(), ((Cylinder) shape).getRadie(), 1000, sub, sub);
			gl.glRotated(-rx, 1.0f, 0.0f, 0.0f);
			gl.glRotated(-ry, 0.0f, 1.0f, 0.0f);

			dir = ((Cylinder) shape).getDirection().negate();
			gl.glRotated(rx, 1.0f, 0.0f, 0.0f);
			gl.glRotated(180 + ry, 0.0f, 1.0f, 0.0f);
			glu.gluCylinder(gluq, ((Cylinder) shape).getRadie(), ((Cylinder) shape).getRadie(), 1000, sub, sub);
		} else if (shape instanceof Blob) {
			for (Vector3d p : ((Blob) shape).getBlobs()) {
				gl.glTranslated(p.x, p.y, p.z);
				glut.glutSolidSphere(((Blob) shape).getRadie(), sub, sub);
				gl.glTranslated(-p.x, -p.y, -p.z);
			}
		} else {
			throw new UnknownShapeException("Primitive: " + shape.getClass().getName());
		}
	}
}
